@@ -201,28 +201,29 @@ class BlueskyMagics(Magics, metaclass=MetaclassForClassProperties):
                 print(label)
                 try:
                     devices = devices_dict[label]
+                    all_children = [(k, getattr(obj, k))
+                                    for _, obj in devices
+                                        for k in getattr(obj, 'read_attrs', [])]
                 except KeyError:
                     print('<no matches for this label>')
                     continue
-                # ignore the first key
-                if are_positioners(devices):
-                    positioners = [positioner[1] for positioner in devices_dict[label]]
+                # Search devices and all their children for positioners.
+                positioners = [dev for _, dev in devices + all_children
+                               if is_positioner(dev)]
+                if positioners:
                     _print_positioners(positioners, precision=self.FMT_PREC,
-                                       prefix=" "*8)
-                else:
-                    _print_devices(devices, prefix=" "*8)
+                                        prefix=" "*2)
+                    print()  # blank line
+                # Just display the top-level devices in the namespace (no
+                # children).
+                _print_devices(devices, prefix=" "*2)
+                print()  # blank line
 
 def _print_devices(devices, prefix=""):
-    cols = ["Python name", "Ophyd Name"]
-    print(prefix + "{:20s} \t {:20s}".format(*cols))
-    print(prefix + "="*40)
+    cols = ["Local variable name", "Ophyd name (to be recorded as metadata)"]
+    print(prefix + "{:38s} {:38s}".format(*cols))
     for name, obj in devices:
-        print(prefix + "{:20s} \t {:20s}".format(name, str(obj.name)))
-
-def are_positioners(devs):
-    # only true if all are positioners
-    # takes ((name, dev),...) tuple
-    return all(is_positioner(obj) for _, obj in devs)
+        print(prefix + "{:38} {:38s}".format(name, str(obj.name)))
 
 def is_positioner(dev):
     return hasattr(dev, 'position')
@@ -260,7 +261,7 @@ def _print_positioners(positioners, sort=True, precision=6, prefix=""):
     for p, v in zip(positioners, values):
         if not isinstance(v, Exception):
             try:
-                prec = p.precision
+                prec = int(p.precision)
             except Exception:
                 prec = precision
             value = np.round(v, decimals=prec)
@@ -312,7 +313,7 @@ def get_labeled_devices(user_ns=None, maxdepth=6):
     obj_list = collections.defaultdict(list)
 
     if maxdepth <= 0:
-        warnings.warn("Recursion limit exceeded")
+        warnings.warn("Recursion limit exceeded. Results will be truncated.")
         return obj_list
 
     if user_ns is None:
@@ -324,16 +325,25 @@ def get_labeled_devices(user_ns=None, maxdepth=6):
         # return of commands)
         # also check its a subclass of desired classes
         if not key.startswith("_"):
-            if is_parent(obj):
-                labels = getattr(obj, '_ophyd_labels_', set())
-                obj_list.update(get_labeled_devices(user_ns=obj.__dict__,
-                                                    maxdepth=maxdepth-1,))
-            else:
-                if hasattr(obj, '_ophyd_labels_'):
-                    # don't inherit parent labels
-                    labels = obj._ophyd_labels_
-                    for label in labels:
-                        obj_list[label].append((key, obj))
+            if hasattr(obj, '_ophyd_labels_'):
+                # don't inherit parent labels
+                labels = obj._ophyd_labels_
+                for label in labels:
+                    obj_list[label].append((key, obj))
+
+                if is_parent(obj):
+                    # Get direct children (not grandchildren).
+                    children = {k: getattr(obj, k)
+                                # obj might be a Signal (no read_attrs).
+                                for k in getattr(obj, 'read_attrs', [])
+                                if '.' not in k}
+                    # Recurse over all children.
+                    for c_key, v in get_labeled_devices(
+                            user_ns=children,
+                            maxdepth=maxdepth-1).items():
+                        items = [('.'.join([key, ot[0]]), ot[1]) for ot in v]
+                        obj_list[c_key].extend(items)
+
 
     # Convert from defaultdict to normal dict before returning.
     return dict(obj_list)
@@ -343,16 +353,7 @@ def is_parent(dev):
     # return whether a node is a parent
     # should not have component_names, or if yes, should be empty
     # read_attrs needed to check it's an instance and not class itself
-    return (isinstance(dev, type) and
-            len(getattr(dev, 'component_names', [])) > 0)
-
-
-def get_children(dev):
-    children = list()
-    if hasattr(dev, 'component_names') and len(dev.component_names) > 0:
-        for comp_name in dev.component_names:
-            children.append(getattr(dev, comp_name))
-    return children
+    return (not isinstance(dev, type) and getattr(dev, 'component_names', []))
 
 
 def _ct_callback(name, doc):
