@@ -1,7 +1,7 @@
 from collections import defaultdict
 from bluesky.run_engine import Msg, RunEngineInterrupted
 from bluesky.examples import stepscan
-from bluesky.plans import (scan, grid_scan, count)
+from bluesky.plans import (scan, grid_scan, count, inner_product_scan)
 from bluesky.object_plans import AbsScanPlan
 from bluesky.preprocessors import run_wrapper, subs_wrapper
 from bluesky.plan_stubs import pause
@@ -10,10 +10,8 @@ from bluesky.callbacks import (CallbackCounter, LiveTable, LiveFit,
                                LiveFitPlot, LivePlot, LiveGrid, LiveScatter)
 from bluesky.callbacks import LiveMesh, LiveRaster  # deprecated but tested
 from bluesky.callbacks.broker import BrokerCallbackBase
-from bluesky.callbacks.zmq import Proxy, Publisher, RemoteDispatcher
-from bluesky.tests.utils import _print_redirect, MsgCollector
-import multiprocessing
-import os
+from bluesky.callbacks import CallbackBase
+from bluesky.tests.utils import _print_redirect, MsgCollector, DocCollector
 import signal
 import threading
 import time
@@ -161,6 +159,12 @@ def test_table(RE, hw):
             assert ln[26:] == kn[26:]
 
 
+def test_table_external(RE, hw, db):
+    RE.subscribe(db.insert)
+    hw.img.reg = db.reg
+    RE(count([hw.img]), LiveTable(['img']))
+
+
 KNOWN_TABLE = """+------------+--------------+----------------+----------------+
 |   seq_num  |        time  |           det  |         motor  |
 +------------+--------------+----------------+----------------+
@@ -240,181 +244,6 @@ KNOWN_TABLE = """+------------+--------------+----------------+----------------+
 +------------+--------------+----------------+----------------+"""
 
 
-def test_zmq(RE, hw):
-    # COMPONENT 1
-    # Run a 0MQ proxy on a separate process.
-    def start_proxy():
-        Proxy(5567, 5568).start()
-
-    proxy_proc = multiprocessing.Process(target=start_proxy, daemon=True)
-    proxy_proc.start()
-    time.sleep(5)  # Give this plenty of time to start up.
-
-    # COMPONENT 2
-    # Run a Publisher and a RunEngine in this main process.
-
-    p = Publisher('127.0.0.1:5567', RE=RE)  # noqa
-
-    # COMPONENT 3
-    # Run a RemoteDispatcher on another separate process. Pass the documents
-    # it receives over a Queue to this process, so we can count them for our
-    # test.
-
-    def make_and_start_dispatcher(queue):
-        def put_in_queue(name, doc):
-            print('putting ', name, 'in queue')
-            queue.put((name, doc))
-
-        d = RemoteDispatcher('127.0.0.1:5568')
-        d.subscribe(put_in_queue)
-        print("REMOTE IS READY TO START")
-        d.loop.call_later(9, d.stop)
-        d.start()
-
-    queue = multiprocessing.Queue()
-    dispatcher_proc = multiprocessing.Process(target=make_and_start_dispatcher,
-                                              daemon=True, args=(queue,))
-    dispatcher_proc.start()
-    time.sleep(5)  # As above, give this plenty of time to start.
-
-    # Generate two documents. The Publisher will send them to the proxy
-    # device over 5567, and the proxy will send them to the
-    # RemoteDispatcher over 5568. The RemoteDispatcher will push them into
-    # the queue, where we can verify that they round-tripped.
-
-    local_accumulator = []
-
-    def local_cb(name, doc):
-        local_accumulator.append((name, doc))
-
-    # Check that numpy stuff is sanitized by putting some in the start doc.
-    md = {'stuff': {'nested': np.array([1, 2, 3])},
-          'scalar_stuff': np.float64(3),
-          'array_stuff': np.ones((3, 3))}
-
-    # RE([Msg('open_run', **md), Msg('close_run')], local_cb)
-    RE(count([hw.det]), local_cb)
-    time.sleep(1)
-
-    # Get the two documents from the queue (or timeout --- test will fail)
-    remote_accumulator = []
-    for i in range(len(local_accumulator)):
-        remote_accumulator.append(queue.get(timeout=2))
-    p.close()
-    proxy_proc.terminate()
-    dispatcher_proc.terminate()
-    proxy_proc.join()
-    dispatcher_proc.join()
-    assert remote_accumulator == local_accumulator
-
-
-def test_zmq_components():
-    # The test `test_zmq` runs Proxy and RemoteDispatcher in a separate
-    # process, which coverage misses.
-    pid = os.getpid()
-
-    def delayed_sigint(delay):
-        time.sleep(delay)
-        os.kill(os.getpid(), signal.SIGINT)
-
-    proxy = Proxy(5567, 5568)
-    assert not proxy.closed
-    threading.Thread(target=delayed_sigint, args=(5,)).start()
-    try:
-        proxy.start()
-        # delayed_sigint stops the proxy
-    except KeyboardInterrupt:
-        ...
-    assert proxy.closed
-    with pytest.raises(RuntimeError):
-        proxy.start()
-
-    proxy = Proxy()  # random port
-    threading.Thread(target=delayed_sigint, args=(5,)).start()
-    try:
-        proxy.start()
-        # delayed_sigint stops the proxy
-    except KeyboardInterrupt:
-        ...
-
-    repr(proxy)
-
-    # test that two ways of specifying address are equivalent
-    d = RemoteDispatcher('localhost:5555')
-    assert d.address == ('localhost', 5555)
-    d = RemoteDispatcher(('localhost', 5555))
-    assert d.address == ('localhost', 5555)
-
-    repr(d)
-
-
-def test_zmq_no_RE(RE):
-    # COMPONENT 1
-    # Run a 0MQ proxy on a separate process.
-    def start_proxy():
-        Proxy(5567, 5568).start()
-
-    proxy_proc = multiprocessing.Process(target=start_proxy, daemon=True)
-    proxy_proc.start()
-    time.sleep(5)  # Give this plenty of time to start up.
-
-    # COMPONENT 2
-    # Run a Publisher and a RunEngine in this main process.
-
-    p = Publisher('127.0.0.1:5567')  # noqa
-
-    # COMPONENT 3
-    # Run a RemoteDispatcher on another separate process. Pass the documents
-    # it receives over a Queue to this process, so we can count them for our
-    # test.
-
-    def make_and_start_dispatcher(queue):
-        def put_in_queue(name, doc):
-            print('putting ', name, 'in queue')
-            queue.put((name, doc))
-
-        d = RemoteDispatcher('127.0.0.1:5568')
-        d.subscribe(put_in_queue)
-        print("REMOTE IS READY TO START")
-        d.loop.call_later(9, d.stop)
-        d.start()
-
-    queue = multiprocessing.Queue()
-    dispatcher_proc = multiprocessing.Process(target=make_and_start_dispatcher,
-                                              daemon=True, args=(queue,))
-    dispatcher_proc.start()
-    time.sleep(5)  # As above, give this plenty of time to start.
-
-    # Generate two documents. The Publisher will send them to the proxy
-    # device over 5567, and the proxy will send them to the
-    # RemoteDispatcher over 5568. The RemoteDispatcher will push them into
-    # the queue, where we can verify that they round-tripped.
-
-    local_accumulator = []
-
-    def local_cb(name, doc):
-        local_accumulator.append((name, doc))
-
-    RE([Msg('open_run'), Msg('close_run')], local_cb)
-
-    # This time the Publisher isn't attached to an RE. Send the documents
-    # manually. (The idea is, these might have come from a Broker instead...)
-    for name, doc in local_accumulator:
-        p(name, doc)
-    time.sleep(1)
-
-    # Get the two documents from the queue (or timeout --- test will fail)
-    remote_accumulator = []
-    for i in range(2):
-        remote_accumulator.append(queue.get(timeout=2))
-    p.close()
-    proxy_proc.terminate()
-    dispatcher_proc.terminate()
-    proxy_proc.join()
-    dispatcher_proc.join()
-    assert remote_accumulator == local_accumulator
-
-
 def test_live_fit(RE, hw):
     try:
         import lmfit
@@ -580,3 +409,43 @@ def test_broker_base_no_unpack(RE, hw, db):
     bc = BrokerChecker(('img',), db=db)
     RE.subscribe(bc)
     RE(count([hw.direct_img]))
+
+
+def test_plotting_hints(RE, hw, db):
+    ''' This tests the run and checks that the correct hints are created.
+        Hints are mainly created to help the BestEffortCallback in plotting the
+        data.
+        Use a callback to do the checking.
+    '''
+    dc = DocCollector()
+    RE.subscribe(dc.insert)
+
+    # check that the inner product hints are passed correctly
+    hint = {'dimensions': [([hw.motor1.name, hw.motor2.name, hw.motor3.name],
+                            'primary')]}
+    RE(inner_product_scan([hw.det], 20, hw.motor1, -1, 1, hw.motor2, -1, 1,
+                          hw.motor3, -2, 0))
+    assert dc.start[-1]['hints'] == hint
+
+    # check that the outer product (grid_scan) hints are passed correctly
+    hint = {'dimensions': [(['motor1'], 'primary'),
+                           (['motor2'], 'primary'),
+                           (['motor3'], 'primary')]}
+    # grid_scan passes "rectilinear" gridding as well
+    # make sure this is also passed
+    output_hint = hint.copy()
+    output_hint['gridding'] = 'rectilinear'
+    RE(grid_scan([hw.det], hw.motor1, -1, 1, 2, hw.motor2, -1, 1, 2,
+                 True, hw.motor3, -2, 0, 2, True))
+
+    assert dc.start[-1]['hints'] == output_hint
+
+    # check that if gridding is supplied, it's not overwritten by grid_scan
+    # check that the outer product (grid_scan) hints are passed correctly
+    hint = {'dimensions': [(['motor1'], 'primary'),
+                           (['motor2'], 'primary'),
+                           (['motor3'], 'primary')],
+            'gridding': 'rectilinear'}
+    RE(grid_scan([hw.det], hw.motor1, -1, 1, 2, hw.motor2, -1, 1, 2,
+                 True, hw.motor3, -2, 0, 2, True))
+    assert dc.start[-1]['hints'] == hint
