@@ -25,6 +25,8 @@ from bluesky.plan_stubs import (
     unstage,
     subscribe,
     unsubscribe,
+    install_suspender,
+    remove_suspender,
     open_run,
     close_run,
     wait_for,
@@ -45,6 +47,7 @@ from bluesky.preprocessors import (
     lazily_stage_wrapper,
     relative_set_wrapper,
     subs_wrapper,
+    suspend_wrapper,
     fly_during_decorator,
     subs_decorator,
     monitor_during_decorator,
@@ -102,6 +105,8 @@ from bluesky.utils import all_safe_rewind
                                                        'func_placeholder',
                                                        'all')]),
      (unsubscribe, (1,), {}, [Msg('unsubscribe', None, token=1)]),
+     (install_suspender, (1,), {}, [Msg('install_suspender', None, 1)]),
+     (remove_suspender, (1,), {}, [Msg('remove_suspender', None, 1)]),
      (open_run, (), {}, [Msg('open_run')]),
      (open_run, (), {'md': {'a': 1}}, [Msg('open_run', a=1)]),
      (close_run, (), {}, [Msg('close_run', reason=None, exit_status=None)]),
@@ -267,6 +272,18 @@ def test_subs():
     assert processed_plan == expected
 
 
+def test_suspend():
+    plan = [Msg('null')]
+
+    processed_plan = list(suspend_wrapper(plan, 1))
+
+    expected = [Msg('install_suspender', None, 1),
+                Msg('null'),
+                Msg('remove_suspender', None, 1)]
+
+    assert processed_plan == expected
+
+
 def test_md():
     def plan():
         yield from open_run(md={'a': 1})
