@@ -1,4 +1,5 @@
 import asyncio
+from event_model import DocumentNames
 import threading
 import types
 import os
@@ -327,7 +328,7 @@ def test_dispatcher_unsubscribe_all(RE):
         pass
 
     RE.subscribe(cb)
-    assert count_callbacks(RE) == 5
+    assert count_callbacks(RE) == len(DocumentNames)
     RE.dispatcher.unsubscribe_all()
     assert count_callbacks(RE) == 0
 
@@ -475,14 +476,14 @@ def _make_unrewindable_suspender_marker():
                  motor,
                  UnReplayableSynGauss('det', motor, 'motor', center=0, Imax=1),
                  ['set', 'trigger', 'sleep',
-                  'rewindable', 'wait_for', 'rewindable',
+                  'rewindable', 'wait_for', 'resume', 'rewindable',
                   'set', 'trigger']))
 
     inps.append((test_plan,
                  motor,
                  SynGauss('det', motor, 'motor', center=0, Imax=1),
                  ['set', 'trigger', 'sleep',
-                  'rewindable', 'wait_for', 'rewindable',
+                  'rewindable', 'wait_for', 'resume', 'rewindable',
                   'set',
                   'trigger', 'sleep', 'set', 'trigger']))
 
@@ -615,6 +616,64 @@ def test_sigint_many_hits_cb(RE):
     assert ttime.time() - start_time < 2
 
 
+def test_no_context_manager(RE):
+    # Clear the context managers so that RE will not react to sigint
+    RE.context_managers = []
+
+    # Proceed as normal
+    pid = os.getpid()
+
+    def sim_kill(n=1):
+        for j in range(n):
+            print('KILL', j)
+            ttime.sleep(0.05)
+            os.kill(pid, signal.SIGINT)
+
+    def hanging_plan():
+        "a plan that blocks the RunEngine's normal Ctrl+C handing with a sleep"
+        ttime.sleep(2)
+        yield Msg('null')
+
+    # Only send one SIGINT
+    timer = threading.Timer(0.5, sim_kill, (1,))
+    timer.start()
+
+    t = threading.Thread(target=RE, args=(hanging_plan()))
+    start = ttime.time()
+    t.start()
+
+    # Wait for the KeyboardInterrupt and handle in the main thread
+    with pytest.raises(KeyboardInterrupt):
+        ttime.sleep(5)
+
+    t.join()
+    delta = ttime.time() - start
+
+    # Hanging plan finished, but extra sleep did not
+    assert 2 < delta < 5
+
+
+def test_many_context_managers(RE):
+    class Manager:
+        enters = 0
+        exits = 0
+
+        def __init__(self, RE):
+            pass
+
+        def __enter__(self):
+            Manager.enters += 1
+
+        def __exit__(self, *args, **kwargs):
+            Manager.exits += 1
+
+    n = 42
+    RE.context_managers.extend([Manager]*n)
+    RE([Msg('null')])
+    assert Manager.enters == n
+    assert Manager.exits == n
+
+
 def _make_plan_marker():
     @reset_positions_decorator()
     def raiser(motor):
@@ -1369,3 +1428,59 @@ def test_drop(RE, hw):
     RE(plan(), collector)
 
     assert len(docs['event']) == 3
+
+
+def test_failing_describe_callback(RE, hw):
+    class TestException(Exception):
+        pass
+    det = hw.det
+    det2 = hw.det2
+
+    def evil_cb(name, doc):
+        if any(k in doc['data_keys'] for k in det.describe()):
+            raise TestException
+
+    RE.subscribe(evil_cb, 'descriptor')
+
+    def plan():
+        yield Msg('open_run')
+        try:
+            yield Msg('create')
+            yield Msg('read', det)
+            yield Msg('save')
+        finally:
+            yield Msg('create')
+            yield Msg('read', det2)
+            yield Msg('save')
+        yield Msg('close_run')
+
+    with pytest.raises(TestException):
+        RE(plan())
+
+def test_print_commands(RE):
+    ''' test the printing of commands available.
+        NOTE : An error here likely just means that this interface has changed.
+        You'll likely want to change the test.
+        (A test here is still a good idea as it at least raises awareness about
+        the changes made breaking past API)
+    '''
+
+    # testing the commands list
+    commands1 = list(RE._command_registry.keys())
+    commands2 = RE.commands
+
+    assert commands1 == commands2
+
+    # testing print commands
+    # copy and paste most of the code...
+    verbose = False
+    print_command_reg1 = "List of available commands\n"
+    for command, func in RE._command_registry.items():
+        docstring = func.__doc__
+        if verbose is False:
+            docstring = docstring.split("\n")[0]
+        print_command_reg1 = print_command_reg1 +\
+            "{} : {}\n".format(command, docstring)
+
+    print_command_reg2 = RE.print_command_registry()
+    assert print_command_reg1 == print_command_reg2
